package weka.confirmationMeasures;

import static weka.confirmationMeasures.ConfirmationMeasuresConstants.*;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

public class ConfirmationMeasureFactory {
	private DivisionByZeroStrategy divisionByZeroStrategy;
	private String className;
	private Constructor<?> measureConstructor;

	public ConfirmationMeasureFactory(String measureName, int divisionByZeroStrategyId)
	{
		switch (divisionByZeroStrategyId) {
		case NAN_VALUE:
			divisionByZeroStrategy = new NanCoercionDivisionByZeroStrategy();
			break;
		case ZERO_VALUE:
			divisionByZeroStrategy = new ZeroCoercionDivisionByZeroStrategy();
			break;
		}
		
		className = "weka.confirmationMeasures." + measureName + "ConfirmationMeasure";
		try {
		Class<?> clazz = Class.forName(className);
		measureConstructor = clazz.getConstructor(DivisionByZeroStrategy.class);
		} catch (ClassNotFoundException | NoSuchMethodException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}
	
	public ConfirmationMeasure createMeasure() {
		try {
			ConfirmationMeasure measure = (ConfirmationMeasure) measureConstructor.newInstance(new Object[] { divisionByZeroStrategy });

			return measure;
		} catch (ClassCastException e) {
			e.printStackTrace();
			throw new RuntimeException("Class " + className
					+ " must extend ConfirmationMeasure class");
		} catch ( InstantiationException
				| IllegalAccessException 
				| IllegalArgumentException
				| InvocationTargetException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}
}
